import os
import sys
from math import fabs

import pygame
from pygame.locals import *

from globals import *
from helper import *
from screen import scr
import tiles
import screen
import map
import spritesheet
import sprite
import menu
import font
import status

# stuff that needs to happen in rough priority order with estimates
# - enemy spawn points
# - dungeon doors 
# - moving platforms 


# - finish menu, text display, etc. 
# - player abilities (dash, high jump, wall grab/jump, dodge roll, &c.) 
# - player robot upgrade menu
# - draw more tiles for set 1 
# - generalize enemy class
# - generalize shot class
# - generalize item class
# - more enemies
# - healing items
# - title screen / main menu? 
# - smooth fade in/out

# - wasteland backgrounds?
# - optimize scrolling, it currently chews way too much cpu
# 	- overhaul tile rendering to fix?
# - may want to change rendering to eliminate the tile buffer, scroll by blitting and redrawing edges?
#	- would still need a buffer, maybe have two that alternate and copy the current one onto the display

clock = pygame.time.Clock()

def main():
	pygame.init()
	pygame.display.set_mode((320,240), pygame.FULLSCREEN | pygame.HWSURFACE | pygame.DOUBLEBUF)
	#pygame.display.set_mode((320,240))
	pygame.mouse.set_visible(False)
	
	#do_mapedit()
	
	tiles.load_tilesets()
	map.load_maps()
	font.load_fonts()
	spritesheet.load_spritesheets()
	scr.map = map.maps[0]
	
	plr = sprite.Player()
	plr.location = (40, 0)
	scr.scroll_by((-120, -120))
	
	nme = sprite.Enemy()
	nme.location = (60, 30)
	enemies = sprite.XGroup()
	enemies.add(nme)
	
	itam = sprite.Item()
	itam.location = (100, 20)
	itemz = sprite.XGroup()
	itemz.add(itam)
	
	projectiles = sprite.XGroup()
	
	grav = 0.0
	user_input = 0
	jump = 0.0
	
	done = False
	
	while not done:
		
		for event in pygame.event.get():
			if event.type == KEYDOWN:
				if event.key == Key.quit:
					done = True
				elif event.key == pygame.K_s:
					temp = pygame.display.get_surface().copy()
					pygame.image.save(temp, 'scr.TGA')
				elif event.key == pygame.K_m:
					menu.TownMenu(0).go()
			elif event.type == KEYUP:
				pass
		
		for p in projectiles:
			if fabs(p.location[0] - plr.location[0]) > RESX * 2 or fabs(p.location[1] - plr.location[1]) > RESY * 2:
				# too far off screen
				p.kill()
			if p.hostile:
				if p.collides_with(plr):
					p.hit_target(plr)
			else:
				for e in enemies:
					if p.collides_with(e):
						p.hit_target(e)
			p.act()
		
		for i in itemz:
			if i.collides_with(plr):
				i.collected()
			i.act()
		
		for e in enemies:
			if e.collides_with(plr) and e.hp > 0:
				plr.hit(e)
				
			if fabs(e.location[0] - plr.location[0]) > RESX * 2 or fabs(e.location[1] - plr.location[1]) > RESY * 2:
				# too far off screen
				e.kill()
			elif e.wait == 0:
				e.act()
			else:
				e.wait -= 1
			
			projectiles.add(e.children.sprites())
			itemz.add(e.drop.sprites())
		
		old_pos = plr.location
		plr.act(pygame.key.get_pressed())
		projectiles.add(plr.children.sprites())
		scr.scroll_by((plr.location[0] - old_pos[0], plr.location[1] - old_pos[1]))
		
		
		if plr.life <= 0:
			# ur dead
			done = True
		
		scr.draw_tiles()
		plr.draw(scr.get_surface())
		enemies.draw(scr.get_surface())
		projectiles.draw(scr.get_surface())
		itemz.draw(scr.get_surface())
		scr.draw_status_area(plr.life, status.maxlife, plr.ammo, status.maxammo)
		scr.done()
		
		clock.tick(frame_rate)
	
	print "Thanks for playing!"


def do_mapedit():
	tiles.load_tilesets()
	map.load_maps()
	font.load_fonts()
	spritesheet.load_spritesheets()
	if len(map.maps) > 0:
		scr.map = map.maps[0]
		mapnum = 0
	else:
		scr.map = map.new_map()
		mapnum = -1
	
	
	done = False
	tilenum = 61
	set = 0
	
	while not done:
		scroll = (0, 0)
		ctile = 0
		for event in pygame.event.get():
			if event.type == KEYDOWN:
				if event.key == Key.up:
					scroll = (0, -16)
				elif event.key == Key.down:
					scroll = (0, 16)
				elif event.key == Key.left:
					scroll = (-16, 0)
				elif event.key == Key.right:
					scroll = (16, 0)
				elif event.key == Key.select:
					scr.set_at_cursor(tilenum)
				elif event.key == pygame.K_q:
					ctile = -1
				elif event.key == pygame.K_a:
					ctile = -5
				elif event.key == pygame.K_z:
					ctile = -20
				elif event.key == pygame.K_w:
					ctile = 1
				elif event.key == pygame.K_s:
					ctile = 5
				elif event.key == pygame.K_x:
					ctile = 20
				elif event.key == pygame.K_e:
					scr.set_enemy_spawn()
				elif event.key == pygame.K_d:
					scr.set_doorway()
				elif event.key == pygame.K_l:
					scr.inc_feature()
				elif event.key == pygame.K_k:
					scr.dec_feature()
				elif event.key == Key.quit:
					done = True
				elif event.key == pygame.K_p: # p is for pickle = save
					if mapnum == -1:
						mapnum = map.save_new_map()
						scr.map = map.maps[mapnum]
					else:
						map.save_maps()
				elif event.key == pygame.K_n:
					mapnum = -1
					scr.map = map.new_map((40, 30), (60, 55, 50))
					scr.need_refresh = True
				elif event.key == pygame.K_i:
					mapnum = (mapnum - 1) % len(map.maps)
					scr.map = map.maps[mapnum]
					scr.need_refresh = True
				elif event.key == pygame.K_o:
					mapnum = (mapnum + 1) % len(map.maps)
					scr.map = map.maps[mapnum]
					scr.need_refresh = True
		
		if scroll != (0, 0):
			scr.scroll_by(scroll)
		
		tilenum = (tilenum + ctile) % tiles.tilesets[set].count()
		
		scr.draw_tiles()
		scr.draw_features()
		scr.draw_cursor()
		pygame.draw.rect(pygame.display.get_surface(), (255, 0, 255), pygame.Rect(4, 4, 18, 18))
		pygame.display.get_surface().blit(tiles.tilesets[set].surf, (5, 5), tiles.tilesets[set].tile(tilenum))
		
		pygame.display.flip()
		
		clock.tick(frame_rate)
	