import math

import pygame
from pygame.locals import *

import tiles
import map
from helper import * 
from globals import *
import font
import status

scr = None

class Screen:
	
	def __init__(self):
		self.need_refresh = True
		self.shift_from = (0, 0)
		self.shift_to = (0, 0)
		self.tile_buffer = pygame.Surface((RESX + (TILEX * 2), RESY + (TILEY * 2)))
		self.temp_buffer = pygame.Surface((RESX + (TILEX * 2), RESY + (TILEY * 2)))
		self.mapx = 0
		self.mapy = 0
		self.offx = 0
		self.offy = 0
		self.map = None
		self.fade = False
	
	def get_surface(self):
		return pygame.display.get_surface()
	
	def draw_status_area(self, life, maxlife, ammo, maxammo):
		box = pygame.Rect(0, 216, 320, 24)
		pygame.draw.rect(self.get_surface(), (80, 80, 80), box, 0)
		pygame.draw.rect(self.get_surface(), (160, 160, 160), box, 1)
		font.write(font.fontm, "Life", (255, 255, 255), (4, 217))
		font.write(font.fontm, "Ammo", (255, 255, 255), (4, 227))
		pygame.draw.rect(self.get_surface(), (120, 0, 0), pygame.Rect(32, 219, maxlife, 8), 0)
		pygame.draw.rect(self.get_surface(), (0, 0, 120), pygame.Rect(32, 229, maxammo, 8), 0)
		if life > 0:
			pygame.draw.rect(self.get_surface(), (255, 160, 0), pygame.Rect(32, 220, life, 6), 0)
		if ammo > 0:
			pygame.draw.rect(self.get_surface(), (0, 160, 255), pygame.Rect(32, 230, ammo, 6), 0)
		font.write(font.fontm, "Parts:" + str(status.parts), (255, 255, 255), (250, 217))
	
	def fade_out(self):
		self.fade = True
		bg = pygame.display.get_surface().copy()
		shade = pygame.surface.Surface((RESX, RESY), 0, bg)
		shade.fill((0, 0, 0))
		for x in range (15, 256, 16):
			pygame.time.wait(5)
			shade.set_alpha(x)
			self.get_surface().blit(bg, (0,0))
			self.get_surface().blit(shade, (0,0))
			pygame.display.flip()
	
	def fade_in(self):
		self.fade = False
		bg = pygame.display.get_surface().copy()
		shade = pygame.surface.Surface((RESX, RESY), 0, bg)
		shade.fill((0, 0, 0))
		for x in range (240, 0, -16):
			pygame.time.wait(5)
			shade.set_alpha(x)
			self.get_surface().blit(bg, (0,0))
			self.get_surface().blit(shade, (0,0))
			pygame.display.flip()
	
	def done(self):
		if self.fade:
			self.fade_in()
		pygame.display.flip()
	
	def draw_tiles(self):
		# redraw everything
		if (self.need_refresh):
			self.tile_buffer.fill(self.map.bg)
			xloc = self.mapx - 1
			yloc = self.mapy - 1
			for x in range(0, NUMTILESX + 2):
				for y in range(0, NUMTILESY + 2):
					self.draw_tile(self.map.tile((x + xloc, y + yloc)), (x * TILEX, y * TILEY))
			self.need_refresh = False
		
		# shift the tiles and only redraw the edges
		# this doesn't actually work right now :(
		if self.shift_from != (0, 0) or self.shift_to != (0,0):
			self.temp_buffer.blit(self.tile_buffer, self.shift_to, pygame.Rect(self.shift_from, (RESX + TILEX, RESY + TILEY)))
			self.temp_buffer, self.tile_buffer = self.tile_buffer, self.temp_buffer
			
			xloc = self.mapx - 1
			yloc = self.mapy - 1
			
			if self.shift_from[0] > 0:
				x = NUMTILESX + 1
			elif self.shift_to[0] > 0:
				x = 0
			else:
				x = None
			if x != None:
				for y in range(0, 22):
					self.draw_tile(self.map.tile((x + xloc, y + yloc)), (x * TILEX, y * TILEY))
			
			if self.shift_from[1] > 0:
				y = NUMTILESY + 1
			elif self.shift_to[1] > 0:
				y = 0
			else:
				y = None
			if y != None:
				for x in range(0, 22):
					self.draw_tile(self.map.tile((x + xloc, y + yloc)), (x * TILEX, y * TILEY))
			
			self.shift_from = (0, 0)
			self.shift_to = (0, 0)
		
		pygame.display.get_surface().blit(self.tile_buffer, (0, 0), pygame.Rect((self.offx + TILEX, self.offy + TILEY), (RESX, RESY)))
	
	def draw_tile(self, tile, loc):
		self.tile_buffer.blit(tiles.tilesets[0].surf, loc, tiles.tilesets[0].tile(tile))
	
	# don't use this yet, it is broked
	def smallscroll_by(self, dist):
		self.offx += dist[0]
		self.offy += dist[1]
		if self.offx > TILEX:
			self.mapx += 1
			self.offx -= TILEX
			self.shift_from = sum_tuple(self.shift_from, (TILEX, 0))
		elif self.offx < -TILEX:
			self.mapx -= 1
			self.offx += TILEX
			self.shift_to = sum_tuple(self.shift_to, (TILEX, 0))
		
		if self.offy > TILEY:
			self.mapy += 1
			self.offy -= TILEY
			self.shift_from = sum_tuple(self.shift_from, (0, TILEY))
		elif self.offy < -TILEY:
			self.mapy -= 1
			self.offy += TILEY
			self.shift_to = sum_tuple(self.shift_to, (0, TILEY))
	
	def scroll_by(self, dist):
		if dist == (0, 0):
			return
		self.offx += dist[0]
		self.offy += dist[1]
		if self.offx > TILEX:
			self.mapx += self.offx / TILEX
			self.offx %= TILEX
		elif self.offx < -TILEX:
			self.mapx -= -self.offx / TILEX
			self.offx = -(-self.offx % TILEX)
		
		if self.offy > TILEY:
			self.mapy += self.offy / TILEY
			self.offy %= TILEY
		elif self.offy < -TILEY:
			self.mapy -= -self.offy / TILEY
			self.offy = -(-self.offy % TILEY)
		self.need_refresh = True
	
	def scroll_to(self, location):
		self.mapx = location[0]
		self.mapy = location[1]
		self.offx = 0
		self.offy = 0
	
	def center_on(self, location):
		self.mapx = location[0] - NUMTILESX / 2
		self.mapy = location[1] - NUMTILESY / 2
	
	def screen_position(self, map_loc):
		return (map_loc[0] - (self.mapx * TILEX + self.offx), map_loc[1] - (self.mapy * TILEY + self.offy))
	
	def draw_cursor(self):
		pygame.draw.rect(pygame.display.get_surface(), (255, 255, 255), pygame.Rect(CURSORX, CURSORY, TILEX, TILEY), 1)

	def set_at_cursor(self, tile):
		self.map.set_tile(self.get_cursor_loc(), tile)
		self.need_refresh = True
	
	def get_cursor_loc(self):
		cx = NUMTILESX / 2 + (self.mapx * TILEX + self.offx) / TILEX
		cy = NUMTILESY / 2 + (self.mapy * TILEY + self.offy) / TILEY
		return (cx, cy)
	
	def set_enemy_spawn(self):
		pass
	
	def set_doorway(self):
		self.map.add_door(self.get_cursor_loc())
	
	def inc_feature(self):
		pass
	
	def dec_feature(self):
		pass
	
	def draw_features(self):
		for d in self.map.doors:
			loc = (d.location[0] * TILEX, d.location[1] * TILEY)
			if d.target == None:
				text = "DOOR"
			else:
				text = str(d.target.mapnum) + ":" + str(d.target.location) 
			font.write(font.font, text, (255, 255, 255), self.screen_position(loc))


scr = Screen()

