import pygame
from pygame.locals import *

from globals import *
from helper import *
from screen import scr
import screen
import spritesheet
import status
import map

class XSprite(pygame.sprite.Sprite):
	def __init__(self, sheet, size):
		pygame.sprite.Sprite.__init__(self)
		self.sheet = spritesheet.spritesheets[sheet]
		self.size = size
		self.frame = 0
		self.location = (0, 0)
		self.wait = 0
		self.cycle = 5
		self.vx = 0
		self.vy = 0
		self.ax = 0
		self.ay = 0
		self.action = self.ac_nothing
		self.automove = True
		self.dieatwall = False
	
	def set_vel(self, x, y):
		self.vx = x
		self.vy = y
	
	def set_acc(self, x, y):
		self.ax = x
		self.ay = y
	
	def act(self, *params):
		self.action(*params)
		if self.wait != None:
			self.wait = self.cycle
		if self.automove:
			self.vx += self.ax
			self.vy += self.ay
			if not self.move_max((int(self.vx), int(self.vy))):
				if self.dieatwall:
					self.action = self.ac_die
				
			
			#loc = self.get_rectangle().move(self.vx, self.vy)
			#self.location = (loc.left, loc.top)
		
	def ac_nothing(self, *params):
		pass
	
	def ac_die(self, *params):
		self.kill()
	
	def draw(self, surface):
		surface.blit(self.sheet.surf, scr.screen_position(self.location), self.sheet.frame(self.size, self.frame))
	
	def get_rectangle(self):
		return pygame.Rect(self.location, (self.size[0] * TILEX, self.size[1] * TILEY))

	def move(self, delta):
		self.location = sum_tuple(self.location, delta)
	
	def obstructed(self, delta):
		return scr.map.obstructed_in(self.get_rectangle().move(delta))
	
	def can_fall(self):
		return not self.obstructed((0, 1))
	
	def at_ledge(self, dx):
		check = self.get_rectangle().move(dx, 2)
		return (not self.can_fall() and not scr.map.obstructed_at((check.right, check.bottom)))
	
	def move_max(self, delta):
		xhit = True
		yhit = True
		if delta[0] != 0:
			xhit = self.move_x_max(delta[0])
		if delta[1] != 0:
			yhit = self.move_y_max(delta[1])
		return (xhit and yhit)
	
	def move_x_max(self, dx):
		if not self.obstructed((dx, 0)):
			self.move((dx, 0))
			return True
		else:
			if dx < 0:
				inc = -1
			else:
				inc = 1
			for count in range(dx * inc):
				if self.obstructed((inc, 0)):
					return False
				else:
					self.move((inc, 0))
	
	def move_y_max(self, dy):
		if not self.obstructed((0, dy)):
			self.move((0, dy))
			return True
		else:
			if dy < 0:
				inc = -1
			else:
				inc = 1
			for count in range(dy * inc):
				if self.obstructed((0, inc)):
					return False
				else:
					self.move((0, inc))
	
	def collides_with(self, other):
		r1 = self.get_rectangle()
		r2 = other.get_rectangle()
		return self.hitbox.move(r1.left, r1.top).colliderect(other.hitbox.move(r2.left, r2.top)) 

class XGroup(pygame.sprite.Group):
	def __init__(self, *spr):
		pygame.sprite.Group.__init__(self, *spr)
	
	def draw(self, surface):
		for spr in self.sprites():
			spr.draw(surface)

class Player(XSprite):
	def __init__(self):
		# the sprite sheet here is never used btw
		XSprite.__init__(self, 'P01', (1,2))
		self.torso = spritesheet.spritesheets['PT1']
		self.bottom = spritesheet.spritesheets['PB1']
		self.left = spritesheet.spritesheets['PL1']
		self.right = spritesheet.spritesheets['PR1']
		self.action = self.ac_start
		self.hitbox = pygame.Rect(2, 0, 14, 32)
		self.life = status.maxlife
		self.ammo = status.maxammo
		self.lastdir = 1
		self.framedir = 1
		self.shotdelay = 0
		self.children = XGroup()
		self.life_regen = 0
		self.ammo_regen = 0
		self.framecount = 0
		self.noenter = False

	def draw(self, surface):
		if self.lastdir > 0:
			surface.blit(self.left.surf, scr.screen_position(self.location), self.left.frame(self.size, self.frame + self.framedir))
			surface.blit(self.torso.surf, scr.screen_position(self.location), self.torso.frame(self.size, self.frame + self.framedir))
			surface.blit(self.bottom.surf, scr.screen_position(self.location), self.bottom.frame(self.size, self.frame + self.framedir))
			surface.blit(self.right.surf, scr.screen_position(self.location), self.right.frame(self.size, self.frame + self.framedir))
		else:
			surface.blit(self.right.surf, scr.screen_position(self.location), self.right.frame(self.size, self.frame + self.framedir))
			surface.blit(self.torso.surf, scr.screen_position(self.location), self.torso.frame(self.size, self.frame + self.framedir))
			surface.blit(self.bottom.surf, scr.screen_position(self.location), self.bottom.frame(self.size, self.frame + self.framedir))
			surface.blit(self.left.surf, scr.screen_position(self.location), self.left.frame(self.size, self.frame + self.framedir))
			
	def act(self, *params):
		self.framecount = (self.framecount + 1) % 12600
		self.life_regen += 1
		if self.life_regen > status.life_regen_rate:
			self.life_regen = 0
			if self.life < status.maxlife:
				self.life += 1
		self.ammo_regen += 1
		if self.ammo_regen > status.ammo_regen_rate:
			self.ammo_regen = 0
			if self.ammo < status.maxammo:
				self.ammo += 1
		if self.shotdelay > 0:
			self.shotdelay -= 1
		XSprite.act(self, *params)
	
	def ac_start(self, keys):
		if self.can_fall():
			self.action = self.ac_fall
		elif keys[Key.down]:
			self.action = self.ac_crouch
		elif keys[Key.jump]:
			self.jumptime = 2
			self.action = self.ac_jump
		elif keys[Key.up]:
			door = scr.map.door_at(self.location)
			if door != None and not self.noenter:
				scr.fade_out()
				scr.map = map.maps[door.target.mapnum]
				#scr.center_on(door.target.location)
				self.location = (door.target.location[0] * TILEX, (door.target.location[1] - 1)* TILEY)
				self.noenter = True
		else:
			self.noenter = False
		
		self.move_on_input(keys)
		if self.vx != 0:
			self.framecount %= 20
			self.frame = 60 + ((self.framecount / 5) * 2)
			if keys[Key.shoot]:
				self.frame += 20
		else:
			self.framecount %= 300
			if keys[Key.shoot]:
				self.frame = 24
			elif self.framecount in range(50, 55) or self.framecount in range(130, 135):
				self.frame = 4
			elif self.framecount in range(260, 295):
				self.frame = 2
			else:
				self.frame = 0
		
		self.shoot_on_input(keys)
		
	def ac_crouch(self, keys):
		self.vx = 0
		if not keys[Key.down]:
			self.action = self.ac_start
		
		if keys[Key.left]:
			self.lastdir = -1
			self.framedir = 0
		elif keys[Key.right]:
			self.lastdir = 1
			self.framedir = 1
		
		if keys[Key.shoot]:
			self.frame = 8
		else:
			self.frame = 6
		
		self.shoot_on_input(keys, True)
	
	def ac_fall(self, keys):
		if self.vy < 0:
				self.frame = 20
		else:
			self.frame = 22
		
		if not self.can_fall():
			self.action = self.ac_start
		
		if self.vy < TERM_VEL:
			self.vy += GRAVITY
		
		if keys[Key.shoot]:
			self.frame += 20
		
		self.move_on_input(keys)
		self.shoot_on_input(keys)			
	
	def ac_jump(self, keys):
		self.vy = -8
		self.frame = 20
		self.jumptime -= 1
		if not keys[Key.jump] or self.jumptime <= 0:
			self.action = self.ac_fall
		
		if keys[Key.shoot]:
			self.frame += 20
		
		self.shoot_on_input(keys)
		self.move_on_input(keys)
	
	def ac_ouch(self, keys):
		self.frame = 44 + ((self.framecount % 2) * 2)
		if self.can_fall() and self.vy < TERM_VEL:
			self.vy += GRAVITY
		elif not self.can_fall() and keys[Key.jump]:
			self.vy = -8
		self.move_on_input(keys)
		
		self.vx += self.ouchdir * (player_speed + 1)
		
		if self.framecount >= 4:
			self.action = self.ac_start

	def move_on_input(self, keys):
		if keys[Key.left]:
			self.vx = -player_speed
			self.lastdir = -1
			self.framedir = 0
		elif keys[Key.right]:
			self.vx = player_speed
			self.lastdir = 1
			self.framedir = 1
		else:
			self.vx = 0
	
	def shoot_on_input(self, keys, crouch = False):
		if keys[Key.shoot] and self.shotdelay <= 0:
			self.ammo -= 1
			self.shotdelay = status.shot_timeout
			shot = Shot((self.lastdir * 6, 0))
			if not crouch:
				shot.location = sum_tuple(self.location, (12 * self.lastdir, 8))
			else:
				shot.location = sum_tuple(self.location, (12 * self.lastdir, 15))
			self.children.add(shot)
			return True
		return False
	
	def hit(self, enemy):
		self.life -= enemy.damage
		if enemy.location[0] > self.location[0]:
			self.ouchdir = -1
		else:
			self.ouchdir = 1
		
		if self.action == self.ac_ouch:
			self.framecount -= 2
		else:
			self.framecount = 0
			self.frame = 44
			self.action = self.ac_ouch

class Shot(XSprite):
	def __init__(self, delta):
		XSprite.__init__(self, 'S01', (1,1))
		self.hitbox = pygame.Rect(0, 6, 16, 4)
		self.damage = 2
		self.action = self.ac_nothing
		self.hostile = False
		self.fade = 320
		self.vx = delta[0]
		self.vy = delta[1]
		self.dieatwall = True
		if self.vx > 0:
			self.frame = 1
		else:
			self.frame = 0
	
	def act(self, *params):
		self.fade -= 1
		if self.fade == 0:
			self.action = self.ac_die
		XSprite.act(self, *params)
	
	def hit_target(self, target):
		target.hit(self.damage)
		self.action = self.ac_die

class Item(XSprite):
	def __init__(self):
		XSprite.__init__(self, 'S01', (1,1))
		self.hitbox = pygame.Rect(0, 0, 16, 16)
		self.framecount = 0
		self.action = self.ac_stuff
		self.vy = 2
	
	def collected(self):
		status.parts += 1
		self.action = self.ac_die
	
	def ac_stuff(self):
		self.framecount += 1
		if self.framecount < 200:
			self.frame = 20 + (self.framecount / 4 ) % 2
		elif self.framecount < 300:
			self.frame = 20 + ((self.framecount / 4 ) % 2) * 2
		else:
			self.action = self.ac_die


class Enemy(XSprite):
	def __init__(self):
		XSprite.__init__(self, 'E01', (1,1))
		self.action = self.ac_start
		self.hitbox = pygame.Rect(0, 8, 16, 8)
		self.damage = 2
		self.hp = 8
		self.children = XGroup()
		self.drop = XGroup()
	
	def ac_start(self):
		self.cycle = 10
		self.set_vel(4, 0)
		self.set_acc(0, 0)
		self.action = self.ac_walk
	
	def ac_walk(self):
		if self.can_fall():
			self.cycle = 0
			self.action = self.ac_fall
		self.frame = (self.frame + 20) % 80
		
		if self.obstructed((self.vx, 0)) or self.at_ledge(self.vx):
			self.vx = -self.vx
	
	def ac_fall(self):
		if self.vy < TERM_VEL:
			self.set_vel(0, self.vy + GRAVITY)
		self.move_y_max(int(self.vy))
		if not self.can_fall():
			self.action = self.ac_start
	
	def ac_ouch(self):
		self.cycle = 10
		self.frame = 0
		self.action = self.ac_walk
	
	def hit(self, ouch):
		self.hp -= ouch
		if self.hp <= 0:
			self.action = self.ac_die
		else:
			self.cycle = 2
			self.frame = 80
			self.action = self.ac_ouch
	
	def ac_die(self):
		itam = Item()
		itam.location = self.location
		self.drop.add(itam)
		XSprite.ac_die(self)

	
	










